# -*- coding: utf-8 -*-

#######################################################################
# Copyright © 2007 Yuri Malheiros.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published
# the Free Software Foundation; version 2 only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#######################################################################

"""
This module implements the trigger of "ctrl+shift+w".
"""

shortcut = "ctrl+shift+w"
sticky = False

class SelectWord(object):
    
    def __init__(self, document_manager):
        """
        Constructor.
        
        @param document_manager: Reference to the DocumentManager.
        @type document_manager: A DocumentManager object.
        """
        
        self.document_manager = document_manager
    
    def activate(self):
        """
        Trigger activator.
        """
        
        buffer = self.document_manager.get_active_view().buffer
        
        insert_mark = buffer.get_insert()
        start = buffer.get_iter_at_mark(insert_mark)
        end = start.copy()
        self.__get_word_bounds(start, end)
        buffer.select_range(end, start)
        
        return True
        
    def __get_word_bounds(self, begin, end):
        if begin.starts_word():
            end.forward_word_end()
        elif end.ends_word():
            begin.backward_word_start()
        elif begin.inside_word():
            begin.backward_word_start()
            end.forward_word_end()
                       
        return (begin, end)
