# -*- coding: utf-8 -*-

#######################################################################
# Copyright © 2007 Yuri Malheiros.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published
# the Free Software Foundation; version 2 only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#######################################################################

"""
This module implements the trigger of "shift+tab".
"""

shortcut = "shift+" + u'\ufe20'
sticky = False

class ShiftTab(object):
    
    def __init__(self, document_manager):
        """
        Constructor.
        
        @param document_manager: Reference to the DocumentManager.
        @type document_manager: A DocumentManager object.
        """
        self.document_manager = document_manager
        
    def activate(self):
        """
        Operations after trigger activation.
        """
        view = self.document_manager.get_active_view()
        buffer = view.buffer
        snippets = view.snippets
        
        if len(snippets.stack) > 0:
            insert = buffer.get_insert()
            
            if snippets.valid_cursor_position():
                for i in range(len(snippets.stack)):
                    try:
                        snippets.previous_field()
                    except KeyError:
                        ins_iter = buffer.get_iter_at_mark(insert)
                        buffer.select_range(ins_iter, ins_iter)
                        snippets.pop_snippet()
                    else:
                        break
                    
            else:
                snippets.pop_snippet()
            return True
            
        else:        
            return False

