# -*- coding: utf-8 -*-

#######################################################################
# Copyright © 2007 Yuri Malheiros.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published
# the Free Software Foundation; version 2 only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#######################################################################

"""
This module implements the trigger of "ctrl+u".
"""

shortcut = "ctrl+u"
sticky = False

class UpperCase(object):
    
    def __init__(self, document_manager):
        """
        Constructor.
        
        @param document_manager: Reference to the DocumentManager.
        @type document_manager: A DocumentManager object.
        """
        
        self.document_manager = document_manager
    
    def activate(self):
        """
        Snippet active.
        """
        
        buffer = self.document_manager.get_active_view().buffer
        
        if buffer.get_has_selection():
            start, end = buffer.get_selection_bounds()
            text = unicode(buffer.get_text(start, end))
            buffer.begin_user_action()
            buffer.delete(start, end)
            buffer.insert_at_cursor(text.upper())
            buffer.end_user_action()
        else:
            insert_mark = buffer.get_insert()
            start = buffer.get_iter_at_mark(insert_mark)
            index = start.get_line_offset()
            end = start.copy()
            self.__get_word_bounds(start, end)
            text = unicode(buffer.get_text(start, end))
            buffer.begin_user_action()
            buffer.delete(start, end)
            buffer.insert_at_cursor(text.upper())
            
            new_insert_iter = buffer.get_iter_at_mark(insert_mark)
            new_insert_iter.set_line_offset(index)
            buffer.place_cursor(new_insert_iter)
            
            buffer.end_user_action()

        return True
        
    def __get_word_bounds(self, begin, end):
        if begin.starts_word():
            end.forward_word_end()
        elif end.ends_word():
            begin.backward_word_start()
        elif begin.inside_word():
            begin.backward_word_start()
            end.forward_word_end()
                       
        return (begin, end)

